"""Unit tests for sagemaker-supported APIs."""

import datetime

import boto3

from moto import mock_aws
from moto.core import DEFAULT_ACCOUNT_ID as ACCOUNT_ID

# See our Development Tips on writing tests for hints on how to write good tests:
# http://docs.getmoto.org/en/latest/docs/contributing/development_tips/tests.html


@mock_aws
def test_create_hyper_parameter_tuning_job():
    client = boto3.client("sagemaker", region_name="us-east-2")
    resp = client.create_hyper_parameter_tuning_job(
        HyperParameterTuningJobName="testhyperparametertuningjob",
        HyperParameterTuningJobConfig={
            "Strategy": "Bayesian",
            "ResourceLimits": {
                "MaxNumberOfTrainingJobs": 123,
                "MaxParallelTrainingJobs": 123,
                "MaxRuntimeInSeconds": 123,
            },
        },
    )

    assert (
        resp["HyperParameterTuningJobArn"]
        == "arn:aws:sagemaker:us-east-2:123456789012:hyper-parameter-tuning-job/testhyperparametertuningjob"
    )


@mock_aws
def test_describe_hyper_parameter_tuning_job_defaults():
    client = boto3.client("sagemaker", region_name="us-east-2")
    client.create_hyper_parameter_tuning_job(
        HyperParameterTuningJobName="testhyperparametertuningjob",
        HyperParameterTuningJobConfig={
            "Strategy": "Bayesian",
            "ResourceLimits": {
                "MaxNumberOfTrainingJobs": 123,
                "MaxParallelTrainingJobs": 123,
                "MaxRuntimeInSeconds": 123,
            },
        },
    )
    resp = client.describe_hyper_parameter_tuning_job(
        HyperParameterTuningJobName="testhyperparametertuningjob"
    )
    assert resp["HyperParameterTuningJobName"] == "testhyperparametertuningjob"
    assert (
        resp["HyperParameterTuningJobArn"]
        == "arn:aws:sagemaker:us-east-2:123456789012:hyper-parameter-tuning-job/testhyperparametertuningjob"
    )
    assert resp["HyperParameterTuningJobConfig"] == {
        "Strategy": "Bayesian",
        "ResourceLimits": {
            "MaxNumberOfTrainingJobs": 123,
            "MaxParallelTrainingJobs": 123,
            "MaxRuntimeInSeconds": 123,
        },
    }
    assert resp["HyperParameterTuningJobStatus"] == "Completed"
    assert isinstance(resp["CreationTime"], datetime.datetime)
    assert isinstance(resp["HyperParameterTuningEndTime"], datetime.datetime)
    assert isinstance(resp["LastModifiedTime"], datetime.datetime)
    assert resp["TrainingJobStatusCounters"] == {
        "Completed": 1,
        "InProgress": 0,
        "NonRetryableError": 0,
        "RetryableError": 0,
        "Stopped": 0,
    }
    assert resp["ObjectiveStatusCounters"] == {
        "Failed": 0,
        "Pending": 0,
        "Succeeded": 1,
    }
    assert resp["BestTrainingJob"] == {
        "TrainingJobDefinitionName": "string",
        "TrainingJobName": "FakeTrainingJobName",
        "TrainingJobArn": "FakeTrainingJobArn",
        "TuningJobName": "FakeTuningJobName",
        "CreationTime": datetime.datetime(2024, 1, 1),
        "TrainingStartTime": datetime.datetime(2024, 1, 1),
        "TrainingEndTime": datetime.datetime(2024, 1, 1),
        "TrainingJobStatus": "Completed",
        "TunedHyperParameters": {"string": "TunedHyperParameters"},
        "FailureReason": "string",
        "FinalHyperParameterTuningJobObjectiveMetric": {
            "Type": "Maximize",
            "MetricName": "Accuracy",
            "Value": 1,
        },
        "ObjectiveStatus": "Succeeded",
    }
    assert resp["OverallBestTrainingJob"] == {
        "TrainingJobDefinitionName": "FakeTrainingJobDefinitionName",
        "TrainingJobName": "FakeTrainingJobName",
        "TrainingJobArn": "FakeTrainingJobArn",
        "TuningJobName": "FakeTuningJobName",
        "CreationTime": datetime.datetime(2024, 1, 1),
        "TrainingStartTime": datetime.datetime(2024, 1, 1),
        "TrainingEndTime": datetime.datetime(2024, 1, 1),
        "TrainingJobStatus": "Completed",
        "TunedHyperParameters": {"string": "FakeTunedHyperParameters"},
        "FailureReason": "FakeFailureReason",
        "FinalHyperParameterTuningJobObjectiveMetric": {
            "Type": "Maximize",
            "MetricName": "Acccuracy",
            "Value": 1,
        },
        "ObjectiveStatus": "Succeeded",
    }
    assert resp["FailureReason"] == ""
    assert resp["TuningJobCompletionDetails"] == {
        "NumberOfTrainingJobsObjectiveNotImproving": 123,
        "ConvergenceDetectedTime": datetime.datetime(2024, 1, 1),
    }
    assert resp["ConsumedResources"] == {"RuntimeInSeconds": 123}


@mock_aws
def test_describe_hyper_parameter_tuning_job():
    client = boto3.client("sagemaker", region_name="us-east-2")
    client.create_hyper_parameter_tuning_job(
        HyperParameterTuningJobName="testhyperparametertuningjob",
        HyperParameterTuningJobConfig={
            "Strategy": "Bayesian",
            "ResourceLimits": {
                "MaxNumberOfTrainingJobs": 123,
                "MaxParallelTrainingJobs": 123,
                "MaxRuntimeInSeconds": 123,
            },
            "StrategyConfig": {
                "HyperbandStrategyConfig": {"MinResource": 123, "MaxResource": 123}
            },
            "HyperParameterTuningJobObjective": {
                "Type": "Maximize",
                "MetricName": "string",
            },
            "ParameterRanges": {
                "IntegerParameterRanges": [
                    {
                        "Name": "string",
                        "MinValue": "string",
                        "MaxValue": "string",
                        "ScalingType": "Auto",
                    },
                ],
            },
            "TrainingJobEarlyStoppingType": "Auto",
            "TuningJobCompletionCriteria": {
                "TargetObjectiveMetricValue": 123,
                "BestObjectiveNotImproving": {
                    "MaxNumberOfTrainingJobsNotImproving": 123
                },
                "ConvergenceDetected": {"CompleteOnConvergence": "Enabled"},
            },
            "RandomSeed": 123,
        },
        TrainingJobDefinition={
            "DefinitionName": "string",
            "TuningObjective": {"Type": "Maximize", "MetricName": "string"},
            "HyperParameterRanges": {
                "IntegerParameterRanges": [
                    {
                        "Name": "string",
                        "MinValue": "string",
                        "MaxValue": "string",
                        "ScalingType": "Auto",
                    },
                ],
                "ContinuousParameterRanges": [
                    {
                        "Name": "string",
                        "MinValue": "string",
                        "MaxValue": "string",
                        "ScalingType": "Auto",
                    },
                ],
                "CategoricalParameterRanges": [
                    {
                        "Name": "string",
                        "Values": [
                            "string",
                        ],
                    },
                ],
                "AutoParameters": [
                    {"Name": "string", "ValueHint": "string"},
                ],
            },
            "StaticHyperParameters": {"string": "string"},
            "AlgorithmSpecification": {
                "TrainingImage": "string",
                "TrainingInputMode": "Pipe",
                "AlgorithmName": "string",
                "MetricDefinitions": [
                    {"Name": "string", "Regex": "string"},
                ],
            },
            "RoleArn": f"arn:aws:iam::{ACCOUNT_ID}:role/FakeRole",
            "InputDataConfig": [
                {
                    "ChannelName": "string",
                    "DataSource": {
                        "S3DataSource": {
                            "S3DataType": "S3Prefix",
                            "S3Uri": "string",
                            "S3DataDistributionType": "FullyReplicated",
                            "AttributeNames": [
                                "string",
                            ],
                            "InstanceGroupNames": [
                                "string",
                            ],
                        },
                        "FileSystemDataSource": {
                            "FileSystemId": "testfilesystemid",
                            "FileSystemAccessMode": "rw",
                            "FileSystemType": "EFS",
                            "DirectoryPath": "string",
                        },
                    },
                    "ContentType": "string",
                    "CompressionType": "Gzip",
                    "RecordWrapperType": "RecordIO",
                    "InputMode": "Pipe",
                    "ShuffleConfig": {"Seed": 123},
                },
            ],
            "VpcConfig": {
                "SecurityGroupIds": [
                    "string",
                ],
                "Subnets": [
                    "string",
                ],
            },
            "OutputDataConfig": {
                "KmsKeyId": "string",
                "S3OutputPath": "string",
                "CompressionType": "GZIP",
            },
            "ResourceConfig": {
                "InstanceType": "ml.m4.xlarge",
                "InstanceCount": 123,
                "VolumeSizeInGB": 123,
                "VolumeKmsKeyId": "string",
                "KeepAlivePeriodInSeconds": 123,
                "InstanceGroups": [
                    {
                        "InstanceType": "ml.m4.xlarge",
                        "InstanceCount": 123,
                        "InstanceGroupName": "string",
                    },
                ],
            },
            "HyperParameterTuningResourceConfig": {
                "InstanceType": "ml.m4.xlarge",
                "InstanceCount": 123,
                "VolumeSizeInGB": 123,
                "VolumeKmsKeyId": "string",
                "AllocationStrategy": "Prioritized",
                "InstanceConfigs": [
                    {
                        "InstanceType": "ml.m4.xlarge",
                        "InstanceCount": 123,
                        "VolumeSizeInGB": 123,
                    },
                ],
            },
            "StoppingCondition": {
                "MaxRuntimeInSeconds": 123,
                "MaxWaitTimeInSeconds": 123,
                "MaxPendingTimeInSeconds": 7200,
            },
            "EnableNetworkIsolation": True,
            "EnableInterContainerTrafficEncryption": True,
            "EnableManagedSpotTraining": True,
            "CheckpointConfig": {"S3Uri": "string", "LocalPath": "string"},
            "RetryStrategy": {"MaximumRetryAttempts": 123},
            "Environment": {"string": "string"},
        },
        TrainingJobDefinitions=[
            {
                "DefinitionName": "string",
                "TuningObjective": {
                    "Type": "Maximize",
                    "MetricName": "string",
                },
                "HyperParameterRanges": {
                    "IntegerParameterRanges": [
                        {
                            "Name": "string",
                            "MinValue": "string",
                            "MaxValue": "string",
                            "ScalingType": "Auto",
                        },
                    ],
                    "ContinuousParameterRanges": [
                        {
                            "Name": "string",
                            "MinValue": "string",
                            "MaxValue": "string",
                            "ScalingType": "Auto",
                        },
                    ],
                    "CategoricalParameterRanges": [
                        {
                            "Name": "string",
                            "Values": [
                                "string",
                            ],
                        },
                    ],
                    "AutoParameters": [
                        {"Name": "string", "ValueHint": "string"},
                    ],
                },
                "StaticHyperParameters": {"string": "string"},
                "AlgorithmSpecification": {
                    "TrainingImage": "string",
                    "TrainingInputMode": "Pipe",
                    "AlgorithmName": "string",
                    "MetricDefinitions": [
                        {"Name": "string", "Regex": "string"},
                    ],
                },
                "RoleArn": f"arn:aws:iam::{ACCOUNT_ID}:role/FakeRole",
                "InputDataConfig": [
                    {
                        "ChannelName": "string",
                        "DataSource": {
                            "S3DataSource": {
                                "S3DataType": "ManifestFile",
                                "S3Uri": "string",
                                "S3DataDistributionType": "FullyReplicated",
                                "AttributeNames": [
                                    "string",
                                ],
                                "InstanceGroupNames": [
                                    "string",
                                ],
                            },
                            "FileSystemDataSource": {
                                "FileSystemId": "testfilesystemid",
                                "FileSystemAccessMode": "rw",
                                "FileSystemType": "EFS",
                                "DirectoryPath": "string",
                            },
                        },
                        "ContentType": "string",
                        "CompressionType": "Gzip",
                        "RecordWrapperType": "RecordIO",
                        "InputMode": "Pipe",
                        "ShuffleConfig": {"Seed": 123},
                    },
                ],
                "VpcConfig": {
                    "SecurityGroupIds": [
                        "string",
                    ],
                    "Subnets": [
                        "string",
                    ],
                },
                "OutputDataConfig": {
                    "KmsKeyId": "string",
                    "S3OutputPath": "string",
                    "CompressionType": "GZIP",
                },
                "ResourceConfig": {
                    "InstanceType": "ml.m4.xlarge",
                    "InstanceCount": 123,
                    "VolumeSizeInGB": 123,
                    "VolumeKmsKeyId": "string",
                    "KeepAlivePeriodInSeconds": 123,
                    "InstanceGroups": [
                        {
                            "InstanceType": "ml.m4.xlarge",
                            "InstanceCount": 123,
                            "InstanceGroupName": "string",
                        },
                    ],
                },
                "HyperParameterTuningResourceConfig": {
                    "InstanceType": "ml.m4.xlarge",
                    "InstanceCount": 123,
                    "VolumeSizeInGB": 123,
                    "VolumeKmsKeyId": "string",
                    "AllocationStrategy": "Prioritized",
                    "InstanceConfigs": [
                        {
                            "InstanceType": "ml.m4.xlarge",
                            "InstanceCount": 123,
                            "VolumeSizeInGB": 123,
                        },
                    ],
                },
                "StoppingCondition": {
                    "MaxRuntimeInSeconds": 123,
                    "MaxWaitTimeInSeconds": 123,
                    "MaxPendingTimeInSeconds": 8000,
                },
                "EnableNetworkIsolation": True,
                "EnableInterContainerTrafficEncryption": True,
                "EnableManagedSpotTraining": True,
                "CheckpointConfig": {"S3Uri": "string", "LocalPath": "string"},
                "RetryStrategy": {"MaximumRetryAttempts": 123},
                "Environment": {"string": "string"},
            },
        ],
        WarmStartConfig={
            "ParentHyperParameterTuningJobs": [
                {"HyperParameterTuningJobName": "string"},
            ],
            "WarmStartType": "IdenticalDataAndAlgorithm",
        },
        Tags=[
            {"Key": "string", "Value": "string"},
        ],
        Autotune={"Mode": "Enabled"},
    )
    resp = client.describe_hyper_parameter_tuning_job(
        HyperParameterTuningJobName="testhyperparametertuningjob"
    )
    assert resp["HyperParameterTuningJobName"] == "testhyperparametertuningjob"
    assert (
        resp["HyperParameterTuningJobArn"]
        == "arn:aws:sagemaker:us-east-2:123456789012:hyper-parameter-tuning-job/testhyperparametertuningjob"
    )
    assert resp["HyperParameterTuningJobConfig"] == {
        "Strategy": "Bayesian",
        "ResourceLimits": {
            "MaxNumberOfTrainingJobs": 123,
            "MaxParallelTrainingJobs": 123,
            "MaxRuntimeInSeconds": 123,
        },
        "StrategyConfig": {
            "HyperbandStrategyConfig": {"MinResource": 123, "MaxResource": 123}
        },
        "HyperParameterTuningJobObjective": {
            "Type": "Maximize",
            "MetricName": "string",
        },
        "ParameterRanges": {
            "IntegerParameterRanges": [
                {
                    "Name": "string",
                    "MinValue": "string",
                    "MaxValue": "string",
                    "ScalingType": "Auto",
                },
            ],
        },
        "TrainingJobEarlyStoppingType": "Auto",
        "TuningJobCompletionCriteria": {
            "TargetObjectiveMetricValue": 123,
            "BestObjectiveNotImproving": {"MaxNumberOfTrainingJobsNotImproving": 123},
            "ConvergenceDetected": {"CompleteOnConvergence": "Enabled"},
        },
        "RandomSeed": 123,
    }
    assert resp["TrainingJobDefinition"] == {
        "DefinitionName": "string",
        "TuningObjective": {"Type": "Maximize", "MetricName": "string"},
        "HyperParameterRanges": {
            "IntegerParameterRanges": [
                {
                    "Name": "string",
                    "MinValue": "string",
                    "MaxValue": "string",
                    "ScalingType": "Auto",
                },
            ],
            "ContinuousParameterRanges": [
                {
                    "Name": "string",
                    "MinValue": "string",
                    "MaxValue": "string",
                    "ScalingType": "Auto",
                },
            ],
            "CategoricalParameterRanges": [
                {
                    "Name": "string",
                    "Values": [
                        "string",
                    ],
                },
            ],
            "AutoParameters": [
                {"Name": "string", "ValueHint": "string"},
            ],
        },
        "StaticHyperParameters": {"string": "string"},
        "AlgorithmSpecification": {
            "TrainingImage": "string",
            "TrainingInputMode": "Pipe",
            "AlgorithmName": "string",
            "MetricDefinitions": [
                {"Name": "string", "Regex": "string"},
            ],
        },
        "RoleArn": f"arn:aws:iam::{ACCOUNT_ID}:role/FakeRole",
        "InputDataConfig": [
            {
                "ChannelName": "string",
                "DataSource": {
                    "S3DataSource": {
                        "S3DataType": "S3Prefix",
                        "S3Uri": "string",
                        "S3DataDistributionType": "FullyReplicated",
                        "AttributeNames": [
                            "string",
                        ],
                        "InstanceGroupNames": [
                            "string",
                        ],
                    },
                    "FileSystemDataSource": {
                        "FileSystemId": "testfilesystemid",
                        "FileSystemAccessMode": "rw",
                        "FileSystemType": "EFS",
                        "DirectoryPath": "string",
                    },
                },
                "ContentType": "string",
                "CompressionType": "Gzip",
                "RecordWrapperType": "RecordIO",
                "InputMode": "Pipe",
                "ShuffleConfig": {"Seed": 123},
            },
        ],
        "VpcConfig": {
            "SecurityGroupIds": [
                "string",
            ],
            "Subnets": [
                "string",
            ],
        },
        "OutputDataConfig": {
            "KmsKeyId": "string",
            "S3OutputPath": "string",
            "CompressionType": "GZIP",
        },
        "ResourceConfig": {
            "InstanceType": "ml.m4.xlarge",
            "InstanceCount": 123,
            "VolumeSizeInGB": 123,
            "VolumeKmsKeyId": "string",
            "KeepAlivePeriodInSeconds": 123,
            "InstanceGroups": [
                {
                    "InstanceType": "ml.m4.xlarge",
                    "InstanceCount": 123,
                    "InstanceGroupName": "string",
                },
            ],
        },
        "HyperParameterTuningResourceConfig": {
            "InstanceType": "ml.m4.xlarge",
            "InstanceCount": 123,
            "VolumeSizeInGB": 123,
            "VolumeKmsKeyId": "string",
            "AllocationStrategy": "Prioritized",
            "InstanceConfigs": [
                {
                    "InstanceType": "ml.m4.xlarge",
                    "InstanceCount": 123,
                    "VolumeSizeInGB": 123,
                },
            ],
        },
        "StoppingCondition": {
            "MaxRuntimeInSeconds": 123,
            "MaxWaitTimeInSeconds": 123,
            "MaxPendingTimeInSeconds": 7200,
        },
        "EnableNetworkIsolation": True,
        "EnableInterContainerTrafficEncryption": True,
        "EnableManagedSpotTraining": True,
        "CheckpointConfig": {"S3Uri": "string", "LocalPath": "string"},
        "RetryStrategy": {"MaximumRetryAttempts": 123},
        "Environment": {"string": "string"},
    }
    assert resp["TrainingJobDefinitions"] == [
        {
            "DefinitionName": "string",
            "TuningObjective": {
                "Type": "Maximize",
                "MetricName": "string",
            },
            "HyperParameterRanges": {
                "IntegerParameterRanges": [
                    {
                        "Name": "string",
                        "MinValue": "string",
                        "MaxValue": "string",
                        "ScalingType": "Auto",
                    },
                ],
                "ContinuousParameterRanges": [
                    {
                        "Name": "string",
                        "MinValue": "string",
                        "MaxValue": "string",
                        "ScalingType": "Auto",
                    },
                ],
                "CategoricalParameterRanges": [
                    {
                        "Name": "string",
                        "Values": [
                            "string",
                        ],
                    },
                ],
                "AutoParameters": [
                    {"Name": "string", "ValueHint": "string"},
                ],
            },
            "StaticHyperParameters": {"string": "string"},
            "AlgorithmSpecification": {
                "TrainingImage": "string",
                "TrainingInputMode": "Pipe",
                "AlgorithmName": "string",
                "MetricDefinitions": [
                    {"Name": "string", "Regex": "string"},
                ],
            },
            "RoleArn": f"arn:aws:iam::{ACCOUNT_ID}:role/FakeRole",
            "InputDataConfig": [
                {
                    "ChannelName": "string",
                    "DataSource": {
                        "S3DataSource": {
                            "S3DataType": "ManifestFile",
                            "S3Uri": "string",
                            "S3DataDistributionType": "FullyReplicated",
                            "AttributeNames": [
                                "string",
                            ],
                            "InstanceGroupNames": [
                                "string",
                            ],
                        },
                        "FileSystemDataSource": {
                            "FileSystemId": "testfilesystemid",
                            "FileSystemAccessMode": "rw",
                            "FileSystemType": "EFS",
                            "DirectoryPath": "string",
                        },
                    },
                    "ContentType": "string",
                    "CompressionType": "Gzip",
                    "RecordWrapperType": "RecordIO",
                    "InputMode": "Pipe",
                    "ShuffleConfig": {"Seed": 123},
                },
            ],
            "VpcConfig": {
                "SecurityGroupIds": [
                    "string",
                ],
                "Subnets": [
                    "string",
                ],
            },
            "OutputDataConfig": {
                "KmsKeyId": "string",
                "S3OutputPath": "string",
                "CompressionType": "GZIP",
            },
            "ResourceConfig": {
                "InstanceType": "ml.m4.xlarge",
                "InstanceCount": 123,
                "VolumeSizeInGB": 123,
                "VolumeKmsKeyId": "string",
                "KeepAlivePeriodInSeconds": 123,
                "InstanceGroups": [
                    {
                        "InstanceType": "ml.m4.xlarge",
                        "InstanceCount": 123,
                        "InstanceGroupName": "string",
                    },
                ],
            },
            "HyperParameterTuningResourceConfig": {
                "InstanceType": "ml.m4.xlarge",
                "InstanceCount": 123,
                "VolumeSizeInGB": 123,
                "VolumeKmsKeyId": "string",
                "AllocationStrategy": "Prioritized",
                "InstanceConfigs": [
                    {
                        "InstanceType": "ml.m4.xlarge",
                        "InstanceCount": 123,
                        "VolumeSizeInGB": 123,
                    },
                ],
            },
            "StoppingCondition": {
                "MaxRuntimeInSeconds": 123,
                "MaxWaitTimeInSeconds": 123,
                "MaxPendingTimeInSeconds": 8000,
            },
            "EnableNetworkIsolation": True,
            "EnableInterContainerTrafficEncryption": True,
            "EnableManagedSpotTraining": True,
            "CheckpointConfig": {"S3Uri": "string", "LocalPath": "string"},
            "RetryStrategy": {"MaximumRetryAttempts": 123},
            "Environment": {"string": "string"},
        },
    ]
    assert resp["HyperParameterTuningJobStatus"] == "Completed"
    assert isinstance(resp["CreationTime"], datetime.datetime)
    assert isinstance(resp["HyperParameterTuningEndTime"], datetime.datetime)
    assert isinstance(resp["LastModifiedTime"], datetime.datetime)
    assert resp["TrainingJobStatusCounters"] == {
        "Completed": 1,
        "InProgress": 0,
        "NonRetryableError": 0,
        "RetryableError": 0,
        "Stopped": 0,
    }
    assert resp["ObjectiveStatusCounters"] == {
        "Failed": 0,
        "Pending": 0,
        "Succeeded": 1,
    }
    assert resp["BestTrainingJob"] == {
        "TrainingJobDefinitionName": "string",
        "TrainingJobName": "FakeTrainingJobName",
        "TrainingJobArn": "FakeTrainingJobArn",
        "TuningJobName": "FakeTuningJobName",
        "CreationTime": datetime.datetime(2024, 1, 1),
        "TrainingStartTime": datetime.datetime(2024, 1, 1),
        "TrainingEndTime": datetime.datetime(2024, 1, 1),
        "TrainingJobStatus": "Completed",
        "TunedHyperParameters": {"string": "TunedHyperParameters"},
        "FailureReason": "string",
        "FinalHyperParameterTuningJobObjectiveMetric": {
            "Type": "Maximize",
            "MetricName": "Accuracy",
            "Value": 1,
        },
        "ObjectiveStatus": "Succeeded",
    }
    assert resp["OverallBestTrainingJob"] == {
        "TrainingJobDefinitionName": "FakeTrainingJobDefinitionName",
        "TrainingJobName": "FakeTrainingJobName",
        "TrainingJobArn": "FakeTrainingJobArn",
        "TuningJobName": "FakeTuningJobName",
        "CreationTime": datetime.datetime(2024, 1, 1),
        "TrainingStartTime": datetime.datetime(2024, 1, 1),
        "TrainingEndTime": datetime.datetime(2024, 1, 1),
        "TrainingJobStatus": "Completed",
        "TunedHyperParameters": {"string": "FakeTunedHyperParameters"},
        "FailureReason": "FakeFailureReason",
        "FinalHyperParameterTuningJobObjectiveMetric": {
            "Type": "Maximize",
            "MetricName": "Acccuracy",
            "Value": 1,
        },
        "ObjectiveStatus": "Succeeded",
    }
    assert resp["WarmStartConfig"] == {
        "ParentHyperParameterTuningJobs": [
            {"HyperParameterTuningJobName": "string"},
        ],
        "WarmStartType": "IdenticalDataAndAlgorithm",
    }
    assert resp["Autotune"] == {"Mode": "Enabled"}
    assert resp["FailureReason"] == ""
    assert resp["TuningJobCompletionDetails"] == {
        "NumberOfTrainingJobsObjectiveNotImproving": 123,
        "ConvergenceDetectedTime": datetime.datetime(2024, 1, 1),
    }
    assert resp["ConsumedResources"] == {"RuntimeInSeconds": 123}


@mock_aws
def test_list_hyper_parameter_tuning_jobs():
    client = boto3.client("sagemaker", region_name="us-east-2")
    client.create_hyper_parameter_tuning_job(
        HyperParameterTuningJobName="testhyperparametertuningjob",
        HyperParameterTuningJobConfig={
            "Strategy": "Bayesian",
            "ResourceLimits": {
                "MaxNumberOfTrainingJobs": 123,
                "MaxParallelTrainingJobs": 123,
                "MaxRuntimeInSeconds": 123,
            },
        },
    )
    client.create_hyper_parameter_tuning_job(
        HyperParameterTuningJobName="testhyperparametertuningjob2",
        HyperParameterTuningJobConfig={
            "Strategy": "Bayesian",
            "ResourceLimits": {
                "MaxNumberOfTrainingJobs": 123,
                "MaxParallelTrainingJobs": 123,
                "MaxRuntimeInSeconds": 123,
            },
        },
    )
    resp = client.list_hyper_parameter_tuning_jobs()["HyperParameterTuningJobSummaries"]
    assert resp[0]["HyperParameterTuningJobName"] == "testhyperparametertuningjob"
    assert (
        resp[0]["HyperParameterTuningJobArn"]
        == "arn:aws:sagemaker:us-east-2:123456789012:hyper-parameter-tuning-job/testhyperparametertuningjob"
    )
    assert resp[0]["HyperParameterTuningJobStatus"] == "Completed"
    assert resp[0]["Strategy"] == "Bayesian"
    assert isinstance(resp[0]["CreationTime"], datetime.datetime)
    assert isinstance(resp[0]["HyperParameterTuningEndTime"], datetime.datetime)
    assert isinstance(resp[0]["LastModifiedTime"], datetime.datetime)
    assert resp[0]["TrainingJobStatusCounters"] == {
        "Completed": 1,
        "InProgress": 0,
        "NonRetryableError": 0,
        "RetryableError": 0,
        "Stopped": 0,
    }
    assert resp[0]["ObjectiveStatusCounters"] == {
        "Failed": 0,
        "Pending": 0,
        "Succeeded": 1,
    }
    assert resp[0]["ResourceLimits"] == {
        "MaxNumberOfTrainingJobs": 123,
        "MaxParallelTrainingJobs": 123,
        "MaxRuntimeInSeconds": 123,
    }
    assert resp[1]["HyperParameterTuningJobName"] == "testhyperparametertuningjob2"
    assert (
        resp[1]["HyperParameterTuningJobArn"]
        == "arn:aws:sagemaker:us-east-2:123456789012:hyper-parameter-tuning-job/testhyperparametertuningjob2"
    )
    assert resp[1]["HyperParameterTuningJobStatus"] == "Completed"
    assert resp[1]["Strategy"] == "Bayesian"
    assert isinstance(resp[1]["CreationTime"], datetime.datetime)
    assert isinstance(resp[1]["HyperParameterTuningEndTime"], datetime.datetime)
    assert isinstance(resp[1]["LastModifiedTime"], datetime.datetime)
    assert resp[1]["TrainingJobStatusCounters"] == {
        "Completed": 1,
        "InProgress": 0,
        "NonRetryableError": 0,
        "RetryableError": 0,
        "Stopped": 0,
    }
    assert resp[1]["ObjectiveStatusCounters"] == {
        "Failed": 0,
        "Pending": 0,
        "Succeeded": 1,
    }
    assert resp[1]["ResourceLimits"] == {
        "MaxNumberOfTrainingJobs": 123,
        "MaxParallelTrainingJobs": 123,
        "MaxRuntimeInSeconds": 123,
    }


@mock_aws
def test_list_hyper_parameter_tuning_jobs_filters():
    client = boto3.client("sagemaker", region_name="us-east-2")
    client.create_hyper_parameter_tuning_job(
        HyperParameterTuningJobName="testhyperparametertuningjob",
        HyperParameterTuningJobConfig={
            "Strategy": "Bayesian",
            "ResourceLimits": {
                "MaxNumberOfTrainingJobs": 123,
                "MaxParallelTrainingJobs": 123,
                "MaxRuntimeInSeconds": 123,
            },
        },
    )
    client.create_hyper_parameter_tuning_job(
        HyperParameterTuningJobName="anothername",
        HyperParameterTuningJobConfig={
            "Strategy": "Bayesian",
            "ResourceLimits": {
                "MaxNumberOfTrainingJobs": 124,
                "MaxParallelTrainingJobs": 122,
                "MaxRuntimeInSeconds": 125,
            },
        },
    )
    client.create_hyper_parameter_tuning_job(
        HyperParameterTuningJobName="testhyperparametertuningjob2",
        HyperParameterTuningJobConfig={
            "Strategy": "Bayesian",
            "ResourceLimits": {
                "MaxNumberOfTrainingJobs": 126,
                "MaxParallelTrainingJobs": 125,
                "MaxRuntimeInSeconds": 128,
            },
        },
    )
    resp = client.list_hyper_parameter_tuning_jobs(
        SortBy="CreationTime",
        SortOrder="Ascending",
        NameContains="testhyperparametertuningjob",
        CreationTimeAfter=datetime.datetime(2024, 1, 1),
        CreationTimeBefore=datetime.datetime(2100, 1, 1),
        LastModifiedTimeAfter=datetime.datetime(2024, 1, 1),
        LastModifiedTimeBefore=datetime.datetime(2100, 1, 1),
        StatusEquals="Completed",
    )["HyperParameterTuningJobSummaries"]
    assert len(resp) == 2
    assert resp[0]["HyperParameterTuningJobName"] == "testhyperparametertuningjob"
    assert (
        resp[0]["HyperParameterTuningJobArn"]
        == "arn:aws:sagemaker:us-east-2:123456789012:hyper-parameter-tuning-job/testhyperparametertuningjob"
    )
    assert resp[0]["HyperParameterTuningJobStatus"] == "Completed"
    assert resp[0]["Strategy"] == "Bayesian"
    assert isinstance(resp[0]["CreationTime"], datetime.datetime)
    assert isinstance(resp[0]["HyperParameterTuningEndTime"], datetime.datetime)
    assert isinstance(resp[0]["LastModifiedTime"], datetime.datetime)
    assert resp[0]["TrainingJobStatusCounters"] == {
        "Completed": 1,
        "InProgress": 0,
        "NonRetryableError": 0,
        "RetryableError": 0,
        "Stopped": 0,
    }
    assert resp[0]["ObjectiveStatusCounters"] == {
        "Failed": 0,
        "Pending": 0,
        "Succeeded": 1,
    }
    assert resp[0]["ResourceLimits"] == {
        "MaxNumberOfTrainingJobs": 123,
        "MaxParallelTrainingJobs": 123,
        "MaxRuntimeInSeconds": 123,
    }
    assert resp[1]["HyperParameterTuningJobName"] == "testhyperparametertuningjob2"
    assert (
        resp[1]["HyperParameterTuningJobArn"]
        == "arn:aws:sagemaker:us-east-2:123456789012:hyper-parameter-tuning-job/testhyperparametertuningjob2"
    )
    assert resp[1]["HyperParameterTuningJobStatus"] == "Completed"
    assert resp[1]["Strategy"] == "Bayesian"
    assert isinstance(resp[1]["CreationTime"], datetime.datetime)
    assert isinstance(resp[1]["HyperParameterTuningEndTime"], datetime.datetime)
    assert isinstance(resp[1]["LastModifiedTime"], datetime.datetime)
    assert resp[1]["TrainingJobStatusCounters"] == {
        "Completed": 1,
        "InProgress": 0,
        "NonRetryableError": 0,
        "RetryableError": 0,
        "Stopped": 0,
    }
    assert resp[1]["ObjectiveStatusCounters"] == {
        "Failed": 0,
        "Pending": 0,
        "Succeeded": 1,
    }
    assert resp[1]["ResourceLimits"] == {
        "MaxNumberOfTrainingJobs": 126,
        "MaxParallelTrainingJobs": 125,
        "MaxRuntimeInSeconds": 128,
    }


@mock_aws
def test_delete_hyper_parameter_tuning_job():
    client = boto3.client("sagemaker", region_name="us-east-2")
    client.create_hyper_parameter_tuning_job(
        HyperParameterTuningJobName="testhyperparametertuningjob",
        HyperParameterTuningJobConfig={
            "Strategy": "Bayesian",
            "ResourceLimits": {
                "MaxNumberOfTrainingJobs": 123,
                "MaxParallelTrainingJobs": 123,
                "MaxRuntimeInSeconds": 123,
            },
        },
    )
    client.create_hyper_parameter_tuning_job(
        HyperParameterTuningJobName="anothername",
        HyperParameterTuningJobConfig={
            "Strategy": "Bayesian",
            "ResourceLimits": {
                "MaxNumberOfTrainingJobs": 124,
                "MaxParallelTrainingJobs": 122,
                "MaxRuntimeInSeconds": 125,
            },
        },
    )
    client.delete_hyper_parameter_tuning_job(HyperParameterTuningJobName="anothername")
    resp = client.list_hyper_parameter_tuning_jobs()["HyperParameterTuningJobSummaries"]
    assert len(resp) == 1
    assert resp[0]["HyperParameterTuningJobName"] == "testhyperparametertuningjob"
    assert (
        resp[0]["HyperParameterTuningJobArn"]
        == "arn:aws:sagemaker:us-east-2:123456789012:hyper-parameter-tuning-job/testhyperparametertuningjob"
    )
    assert resp[0]["HyperParameterTuningJobStatus"] == "Completed"
    assert resp[0]["Strategy"] == "Bayesian"
    assert isinstance(resp[0]["CreationTime"], datetime.datetime)
    assert isinstance(resp[0]["HyperParameterTuningEndTime"], datetime.datetime)
    assert isinstance(resp[0]["LastModifiedTime"], datetime.datetime)
    assert resp[0]["TrainingJobStatusCounters"] == {
        "Completed": 1,
        "InProgress": 0,
        "NonRetryableError": 0,
        "RetryableError": 0,
        "Stopped": 0,
    }
    assert resp[0]["ObjectiveStatusCounters"] == {
        "Failed": 0,
        "Pending": 0,
        "Succeeded": 1,
    }
    assert resp[0]["ResourceLimits"] == {
        "MaxNumberOfTrainingJobs": 123,
        "MaxParallelTrainingJobs": 123,
        "MaxRuntimeInSeconds": 123,
    }


@mock_aws
def test_tag_hyper_parameter_tuning_job():
    client = boto3.client("sagemaker", region_name="us-east-2")
    arn = client.create_hyper_parameter_tuning_job(
        HyperParameterTuningJobName="testhyperparametertuningjob",
        HyperParameterTuningJobConfig={
            "Strategy": "Bayesian",
            "ResourceLimits": {
                "MaxNumberOfTrainingJobs": 123,
                "MaxParallelTrainingJobs": 123,
                "MaxRuntimeInSeconds": 123,
            },
        },
        Tags=[{"Key": "testkey", "Value": "testvalue"}],
    )["HyperParameterTuningJobArn"]

    resp1 = client.list_tags(ResourceArn=arn)["Tags"]
    assert resp1 == [
        {"Key": "testkey", "Value": "testvalue"},
    ]
    client.add_tags(
        ResourceArn=arn,
        Tags=[
            {"Key": "testkey2", "Value": "testvalue2"},
        ],
    )
    resp2 = client.list_tags(ResourceArn=arn)["Tags"]
    assert resp2 == [
        {"Key": "testkey", "Value": "testvalue"},
        {"Key": "testkey2", "Value": "testvalue2"},
    ]
    client.delete_tags(
        ResourceArn=arn,
        TagKeys=[
            "testkey",
        ],
    )
    resp3 = client.list_tags(ResourceArn=arn)["Tags"]
    assert resp3 == [{"Key": "testkey2", "Value": "testvalue2"}]



import json
import sys
from typing import Dict, Optional
from unittest import SkipTest

import boto3
import pytest
from botocore.exceptions import ClientError

import moto.iotdata.models
from moto import mock_aws, settings
from moto.core import DEFAULT_ACCOUNT_ID as ACCOUNT_ID
from moto.utilities.distutils_version import LooseVersion

from . import iot_aws_verified

boto3_version = sys.modules["botocore"].__version__


@iot_aws_verified()
@pytest.mark.aws_verified
def test_basic(name: Optional[str] = None) -> None:
    client = boto3.client("iot-data", region_name="ap-northeast-1")

    raw_payload = b'{"state": {"desired": {"led": "on"}}}'

    with pytest.raises(ClientError):
        client.get_thing_shadow(thingName=name)

    res = client.update_thing_shadow(thingName=name, payload=raw_payload)

    payload = json.loads(res["payload"].read())
    expected_state = '{"desired": {"led": "on"}}'
    assert payload["state"] == json.loads(expected_state)
    assert "led" in payload["metadata"]["desired"]
    assert payload["version"] == 1
    assert "timestamp" in payload

    res = client.get_thing_shadow(thingName=name)
    payload = json.loads(res["payload"].read())
    expected_state = '{"desired": {"led": "on"}, "delta": {"led": "on"}}'
    assert payload["state"] == json.loads(expected_state)
    assert "led" in payload["metadata"]["desired"]
    assert payload["version"] == 1
    assert "timestamp" in payload

    client.delete_thing_shadow(thingName=name)
    with pytest.raises(ClientError):
        client.get_thing_shadow(thingName=name)


@iot_aws_verified()
@pytest.mark.aws_verified
def test_update(name: Optional[str] = None) -> None:
    client = boto3.client("iot-data", region_name="ap-northeast-1")
    raw_payload = b'{"state": {"desired": {"led": "on"}}}'

    # first update
    res = client.update_thing_shadow(thingName=name, payload=raw_payload)
    payload = json.loads(res["payload"].read())
    expected_state = '{"desired": {"led": "on"}}'
    assert payload["state"] == json.loads(expected_state)
    assert "led" in payload["metadata"]["desired"]
    assert payload["version"] == 1
    assert "timestamp" in payload

    res = client.get_thing_shadow(thingName=name)
    payload = json.loads(res["payload"].read())
    expected_state = '{"desired": {"led": "on"}, "delta": {"led": "on"}}'
    assert payload["state"] == json.loads(expected_state)
    assert "led" in payload["metadata"]["desired"]
    assert payload["version"] == 1
    assert "timestamp" in payload

    # reporting new state
    new_payload = b'{"state": {"reported": {"led": "on"}}}'
    res = client.update_thing_shadow(thingName=name, payload=new_payload)
    payload = json.loads(res["payload"].read())
    expected_state = '{"reported": {"led": "on"}}'
    assert payload["state"] == json.loads(expected_state)
    assert "led" in payload["metadata"]["reported"]
    assert payload["version"] == 2
    assert "timestamp" in payload

    res = client.get_thing_shadow(thingName=name)
    payload = json.loads(res["payload"].read())
    expected_state = '{"desired": {"led": "on"}, "reported": {"led": "on"}}'
    assert payload["state"] == json.loads(expected_state)
    assert "led" in payload["metadata"]["desired"]
    assert payload["version"] == 2
    assert "timestamp" in payload

    raw_payload = b'{"state": {"desired": {"led": "on"}}, "version": 1}'
    with pytest.raises(ClientError) as ex:
        client.update_thing_shadow(thingName=name, payload=raw_payload)
    assert ex.value.response["ResponseMetadata"]["HTTPStatusCode"] == 409
    assert ex.value.response["Error"]["Message"] == "Version conflict"


@iot_aws_verified()
@pytest.mark.aws_verified
def test_create_named_shadows(name: Optional[str] = None) -> None:
    if LooseVersion(boto3_version) < LooseVersion("1.29.0"):
        raise SkipTest("Parameter only available in newer versions")
    client = boto3.client("iot-data", region_name="ap-northeast-1")
    thing_name = name

    # default shadow
    default_payload = json.dumps({"state": {"desired": {"name": "default"}}})
    res = client.update_thing_shadow(thingName=thing_name, payload=default_payload)
    payload = json.loads(res["payload"].read())
    assert payload["state"] == {"desired": {"name": "default"}}

    # Create named shadows
    for name in ["shadow1", "shadow2"]:
        named_payload = json.dumps({"state": {"reported": {"name": name}}}).encode(
            "utf-8"
        )
        client.update_thing_shadow(
            thingName=thing_name, payload=named_payload, shadowName=name
        )

        res = client.get_thing_shadow(thingName=thing_name, shadowName=name)
        payload = json.loads(res["payload"].read())
        assert payload["state"]["reported"] == {"name": name}

    # List named shadows
    shadows = client.list_named_shadows_for_thing(thingName=thing_name)["results"]
    assert len(shadows) == 2
    assert "shadow1" in shadows
    assert "shadow2" in shadows

    # Verify we can delete a named shadow
    client.delete_thing_shadow(thingName=thing_name, shadowName="shadow2")

    with pytest.raises(ClientError) as exc:
        client.get_thing_shadow(thingName=thing_name, shadowName="shadow2")
    err = exc.value.response["Error"]
    assert err["Code"] == "ResourceNotFoundException"

    # The default and other named shadow are still there
    assert "payload" in client.get_thing_shadow(thingName=thing_name)
    assert "payload" in client.get_thing_shadow(
        thingName=thing_name, shadowName="shadow1"
    )


@mock_aws
def test_publish() -> None:
    region_name = "ap-northeast-1"
    client = boto3.client("iot-data", region_name=region_name)
    client.publish(topic="test/topic1", qos=1, payload=b"pl1")
    client.publish(topic="test/topic2", qos=1, payload=b"pl2")
    client.publish(topic="test/topic3", qos=1, payload=b"\xbf")
    client.publish(topic="test/topic4", qos=1, payload="string")

    if not settings.TEST_SERVER_MODE:
        mock_backend = moto.iotdata.models.iotdata_backends[ACCOUNT_ID][region_name]
        assert len(mock_backend.published_payloads) == 4
        assert ("test/topic1", b"pl1") in mock_backend.published_payloads
        assert ("test/topic2", b"pl2") in mock_backend.published_payloads
        assert ("test/topic3", b"\xbf") in mock_backend.published_payloads
        assert ("test/topic4", b"string") in mock_backend.published_payloads


@iot_aws_verified()
@pytest.mark.aws_verified
def test_delete_field_from_device_shadow(name: Optional[str] = None) -> None:
    iot = boto3.client("iot-data", region_name="ap-northeast-1")

    iot.update_thing_shadow(
        thingName=name,
        payload=json.dumps({"state": {"desired": {"state1": 1, "state2": 2}}}),
    )
    response = json.loads(iot.get_thing_shadow(thingName=name)["payload"].read())
    assert len(response["state"]["desired"]) == 2

    iot.update_thing_shadow(
        thingName=name,
        payload=json.dumps({"state": {"desired": {"state1": None}}}),
    )
    response = json.loads(iot.get_thing_shadow(thingName=name)["payload"].read())
    assert len(response["state"]["desired"]) == 1
    assert "state2" in response["state"]["desired"]

    iot.update_thing_shadow(
        thingName=name,
        payload=json.dumps({"state": {"desired": {"state2": None}}}),
    )
    response = json.loads(iot.get_thing_shadow(thingName=name)["payload"].read())
    assert "desired" not in response["state"]


@iot_aws_verified()
@pytest.mark.aws_verified
@pytest.mark.parametrize(
    "desired,initial_delta,reported,delta_after_report",
    [
        (
            {"desired": {"online": True}},
            {"desired": {"online": True}, "delta": {"online": True}},
            {"reported": {"online": False}},
            {
                "desired": {"online": True},
                "reported": {"online": False},
                "delta": {"online": True},
            },
        ),
        (
            {"desired": {"enabled": True}},
            {"desired": {"enabled": True}, "delta": {"enabled": True}},
            {"reported": {"online": False, "enabled": True}},
            {
                "desired": {"enabled": True},
                "reported": {"online": False, "enabled": True},
            },
        ),
        ({}, {}, {"reported": {"online": False}}, {"reported": {"online": False}}),
        ({}, {}, {"reported": {"online": None}}, {}),
        (
            {"desired": {}},
            {},
            {"reported": {"online": False}},
            {"reported": {"online": False}},
        ),
    ],
)
def test_delta_calculation(
    desired: Dict[str, Dict[str, Optional[bool]]],
    initial_delta: Dict[str, Dict[str, Optional[bool]]],
    reported: Dict[str, Dict[str, Optional[bool]]],
    delta_after_report: Dict[str, Dict[str, Optional[bool]]],
    name: Optional[str] = None,
) -> None:
    client = boto3.client("iot-data", region_name="ap-northeast-1")
    desired_payload = json.dumps({"state": desired}).encode("utf-8")
    client.update_thing_shadow(thingName=name, payload=desired_payload)

    res = client.get_thing_shadow(thingName=name)
    payload = json.loads(res["payload"].read())
    assert payload["state"] == initial_delta

    reported_payload = json.dumps({"state": reported}).encode("utf-8")
    client.update_thing_shadow(thingName=name, payload=reported_payload)

    res = client.get_thing_shadow(thingName=name)
    payload = json.loads(res["payload"].read())
    assert payload["state"] == delta_after_report



"""Exceptions raised by the bedrockagent service."""

from moto.core.exceptions import JsonRESTError


class AgentsforBedrockClientError(JsonRESTError):
    code = 400


class ResourceNotFoundException(AgentsforBedrockClientError):
    def __init__(self, msg: str):
        super().__init__("ResourceNotFoundException", f"{msg}")


class ConflictException(AgentsforBedrockClientError):
    def __init__(self, msg: str):
        super().__init__("ConflictException", f"{msg}")


class ValidationException(AgentsforBedrockClientError):
    def __init__(self, msg: str):
        super().__init__(
            "ValidationException",
            "Input validation failed. Check your request parameters and retry the request.",
            f"{msg}",
        )



from moto.core.exceptions import RESTError


class ReceiptHandleIsInvalid(RESTError):
    code = 400

    def __init__(self) -> None:
        super().__init__(
            "ReceiptHandleIsInvalid", "The input receipt handle is invalid."
        )


class MessageAttributesInvalid(RESTError):
    code = 400

    def __init__(self, description: str):
        super().__init__("MessageAttributesInvalid", description)


class QueueDoesNotExist(RESTError):
    code = 404

    def __init__(self) -> None:
        super().__init__(
            "AWS.SimpleQueueService.NonExistentQueue",
            "The specified queue does not exist for this wsdl version.",
            template="wrapped_single_error",
        )


class QueueAlreadyExists(RESTError):
    code = 400

    def __init__(self, message: str):
        super().__init__("QueueAlreadyExists", message)


class EmptyBatchRequest(RESTError):
    code = 400

    def __init__(self, action: str = "Send") -> None:
        super().__init__(
            "AWS.SimpleQueueService.EmptyBatchRequest",
            f"There should be at least one {action}MessageBatchRequestEntry in the request.",
        )


class InvalidBatchEntryId(RESTError):
    code = 400

    def __init__(self) -> None:
        super().__init__(
            "InvalidBatchEntryId",
            "A batch entry id can only contain alphanumeric characters, "
            "hyphens and underscores. It can be at most 80 letters long.",
        )


class BatchRequestTooLong(RESTError):
    code = 400

    def __init__(self, length: int):
        super().__init__(
            "BatchRequestTooLong",
            "Batch requests cannot be longer than 262144 bytes. "
            f"You have sent {length} bytes.",
        )


class BatchEntryIdsNotDistinct(RESTError):
    code = 400

    def __init__(self, entry_id: str):
        super().__init__("BatchEntryIdsNotDistinct", f"Id {entry_id} repeated.")


class TooManyEntriesInBatchRequest(RESTError):
    code = 400

    def __init__(self, number: int):
        super().__init__(
            "TooManyEntriesInBatchRequest",
            "Maximum number of entries per request are 10. " f"You have sent {number}.",
        )


class InvalidAttributeName(RESTError):
    code = 400

    def __init__(self, attribute_name: str):
        super().__init__("InvalidAttributeName", f"Unknown Attribute {attribute_name}.")


class InvalidAttributeValue(RESTError):
    code = 400

    def __init__(self, attribute_name: str):
        super().__init__(
            "InvalidAttributeValue",
            f"Invalid value for the parameter {attribute_name}.",
        )


class InvalidParameterValue(RESTError):
    code = 400

    def __init__(self, message: str):
        super().__init__("InvalidParameterValue", message)


class MissingParameter(RESTError):
    code = 400

    def __init__(self, parameter: str):
        super().__init__(
            "MissingParameter", f"The request must contain the parameter {parameter}."
        )


class OverLimit(RESTError):
    code = 403

    def __init__(self, count: int):
        super().__init__(
            "OverLimit", f"{count} Actions were found, maximum allowed is 7."
        )
